import javax.xml.parsers.*;
import org.xml.sax.*;
import org.w3c.dom.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;  
import javax.xml.transform.stream.StreamResult; 
import java.io.*;

public class MathDefDictionary {

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------

  public boolean overwrite = false;
  public boolean print = false;
  
  private String directory = "../defs/";
  private String indexFile = "0index.xml";
  private Document index;
  
  private DocumentBuilderFactory factory;
  private DocumentBuilder builder;
  private TransformerFactory tFactory;
  private Transformer transformer;
  
  private final int textXMLplace = 0;
  private final int definiendumXMLplace = 1;
  private final int definiensXMLplace = 2;

// ----------------------------------------------------------------------------
// Constructors
// ----------------------------------------------------------------------------

  public MathDefDictionary() {
    //Set up the XML parser:
    try {
      factory = DocumentBuilderFactory.newInstance();
      builder = factory.newDocumentBuilder();
      tFactory = TransformerFactory.newInstance();
      transformer = tFactory.newTransformer();
      
    } catch (TransformerConfigurationException tce) {
      // Error generated by the parser
      System.out.println ("\n** Transformer Factory error");
      System.out.println("   " + tce.getMessage() );

      // Use the contained exception, if any
      Throwable x = tce;
      if (tce.getException() != null)
        x = tce.getException();
      x.printStackTrace();
      
    } catch (TransformerException te) {
      // Error generated by the parser
      System.out.println ("\n** Transformation error");
      System.out.println("   " + te.getMessage() );

      // Use the contained exception, if any
      Throwable x = te;
      if (te.getException() != null)
        x = te.getException();
      x.printStackTrace();
           
    } catch (ParserConfigurationException pce) {
      // Parser with specified options can't be built
      pce.printStackTrace();
    }
    //Open the index:
    this.index = openIndex();
  }

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------
 
  public Document getNewDocument() {
    return builder.newDocument();
  }
  
  public void writeXML(Document doc, String fileName) {
  /** ***************************************************************
   *  You pass the Document you want to record as an XML file,
   *  and the filename you want it to be saved as, and this method
   *  saves it in the directory given by this.directory.
   **************************************************************** */
    try {
      DOMSource source = new DOMSource(doc);
      File f = new File(directory + fileName + ".xml");
      StreamResult result = new StreamResult(f);
      transformer.transform(source, result);
    } catch (TransformerConfigurationException tce) {
           // Error generated by the parser
           System.out.println ("\n** Transformer Factory error");
           System.out.println("   " + tce.getMessage() );

           // Use the contained exception, if any
           Throwable x = tce;
           if (tce.getException() != null)
               x = tce.getException();
           x.printStackTrace();
      
    } catch (TransformerException te) {
           // Error generated by the parser
           System.out.println ("\n** Transformation error");
           System.out.println("   " + te.getMessage() );

           // Use the contained exception, if any
           Throwable x = te;
           if (te.getException() != null)
               x = te.getException();
           x.printStackTrace();
           
    } catch (Exception ex) {
           ex.printStackTrace();
    }
  }

/*  
  public boolean recordDef(String filename, String dfd, Body mdb) {
    boolean recorded = false;
    //Determine if the dfd already exists:
    String existingFile = null;
    int columnIn = 1;
    int columnOut = 3;
    existingFile = searchIndex(columnIn, dfd, columnOut);
    if (existingFile==null) { //the dfd does not already exist
      //Create a file for this dfd, and define it for this arity:
      newDfd(filename, dfd, mdb);
      //
      recorded = true;
    } else {
      //The dfd does already exist. Determine if it has yet been
      //defined for the desired arity.
      Document d = parseFileName(existingFile+".xml");
      NodeList n = d.getFirstChild().getChildNodes();
      Element e = null; String s; boolean b = false;
      for (int i = 0; i < n.getLength(); i++) {
        e = (Element) n.item(i);
        s = e.getAttribute("arity");
        if (s.equals(mdb.arity)) {b = true; break;}
      }
      if (!b) { //The dfd has not yet been defined for this arity.
        //Add a definition of this dfd for this arity:
        newArity(existingFile, mdb, d);
        //
        recorded = true;
      } else { //The dfd has already been defined for this arity.
        if (overwrite) {
          //Overwrite the old definition of this dfd for this arity:
          overwriteArity(existingFile, mdb, d, e);
          //
          recorded = true;
        }
      }
    }
    return recorded;
  }
*/  
  public void saveIndex() {
  /** ***************************************************************
   *  Records any and all changes that have been made to the index.
   **************************************************************** */
   try {
      DOMSource source = new DOMSource(index);
      File f = new File(indexFile);
      StreamResult result = new StreamResult(f);
      transformer.transform(source, result);
    } catch (TransformerConfigurationException tce) {
           // Error generated by the parser
           System.out.println ("\n** Transformer Factory error");
           System.out.println("   " + tce.getMessage() );

           // Use the contained exception, if any
           Throwable x = tce;
           if (tce.getException() != null)
               x = tce.getException();
           x.printStackTrace();
      
    } catch (TransformerException te) {
           // Error generated by the parser
           System.out.println ("\n** Transformation error");
           System.out.println("   " + te.getMessage() );

           // Use the contained exception, if any
           Throwable x = te;
           if (te.getException() != null)
               x = te.getException();
           x.printStackTrace();
           
    } catch (Exception ex) {
           ex.printStackTrace();
    }
  }
  
  //should remove arity argument
  public Node lookUpDefiniendum(File inputFile, String arity) {
    Document doc = lookUp(inputFile);
    return lookUpDefiniendum(doc, arity);
  }
  
  public Node lookUpDefiniendum(String dfd, String arity) {
    Document doc = lookUp(dfd);
    return lookUpDefiniendum(doc, arity);
  }
  
  public Node lookUpDefiniendum(Document doc, String arity) {
    Node x = null;
    if (doc!=null) {
      NodeList n = doc.getFirstChild().getChildNodes();
      Element e; String s; int w = -1;
      for (int i = 0; i < n.getLength(); i++) {
        e = (Element) n.item(i);
        s = e.getAttribute("arity");
        if (s.equals(arity)) {
          w = i; break;
        }
      }
      if (w>=0) {
        e = (Element) n.item(w);
        NodeList m = e.getChildNodes();
        x = m.item(definiendumXMLplace);
      }
    }
    return x;
  }
  
  //Should remove arity argument
  public Node lookUpDefiniens(File inputFile, String arity) {
    Document doc = lookUp(inputFile);
    return lookUpDefiniens(doc, arity);
  }
  
  public Node lookUpDefiniens(String dfd, String arity) {
    Document doc = lookUp(dfd);
    //Diagnostic:
    if (doc!=null) {System.out.println("<MathDefDictionary.java:238>"+
                    "Looked up " + dfd + " Document successfully.");}
    //
    return lookUpDefiniens(doc, arity);
  }
  
  public Node lookUpDefiniens(Document doc, String arity) {
    Node x = null;
    if (doc!=null) {
      //Diagnostic
      System.out.println("<MathDefDictionary.java:248>Checkpoint 1.");
      //
      NodeList n = doc.getFirstChild().getChildNodes();
      Element e; String s; int w = -1;
      //Diagnostic:
      System.out.println("<MathDefDictionary.java:253>Searching for arity: "
                         + arity + ".");
      //
      for (int i = 0; i < n.getLength(); i++) {
        e = (Element) n.item(i);
        s = e.getAttribute("arity");
        if (s.equals(arity)) {
          w = i;
          //Diagnostic:
          System.out.println("<MathDefDictionary.java:255>Found arity "
                             + arity + " successfully. It was item number "
                             + Integer.toString(w) + ".");
          //
          break;
        }
      }
      //Diagnostic:
      System.out.println("<MathDefDictionary.java:266>w equals " +
                         Integer.toString(w));
      //
      if (w<0 && arity.equals("2")) {
        //Try again, searching for infix arity.
        arity = "infix";
        for (int i = 0; i < n.getLength(); i++) {
          e = (Element) n.item(i);
          s = e.getAttribute("arity");
          if (s.equals(arity)) {
            w = i;
            //Diagnostic:
            System.out.println("<MathDefDictionary.java:255>Found arity "
                               + arity + " successfully. It was item number "
                               + Integer.toString(w) + ".");
            //
            break;
          }
        }
      }
      //Diagnostic:
      System.out.println("<MathDefDictionary.java:266>w equals " +
                         Integer.toString(w));
      //
      if (w>=0) {
        e = (Element) n.item(w);
        NodeList m = e.getChildNodes();
        x = m.item(definiensXMLplace);
      }
    }
    return x;
  }
  
  public String lookUpInfixPrecedence(String dfd) {
    String p = null;
    Document doc = lookUp(dfd);
    if (doc!=null) {
      NodeList n = doc.getFirstChild().getChildNodes();
      Element e; String s; int w = -1;
      for (int i = 0; i < n.getLength(); i++) {
        e = (Element) n.item(i);
        s = e.getAttribute("arity");
        if (s.equals("infix")) {
          w = i; break;
        }
      }
      if (w>=0) {
        e = (Element) n.item(w);
        p = e.getAttribute("precedence");
      }
    }
    return p;
  }

//---------------------------------------------

  public Document lookUp(File inputFile) {
    Document d = null;
    d = parseFile(inputFile);
    return d;
  }
  
  public Document lookUp(String s) {
    Document d = null;
    String fileName = null;
    int columnIn = 1;
    int columnOut = 3;
    fileName = searchIndex(columnIn, s, columnOut);
    if (fileName!=null) {
      d = parseFileName(fileName+".xml");
    }
    return d;
  }
  
  public void recordIndexEntry(String name, String fileName) {
  /** ***************************************************************
   *  You give the name and filename of a new entry to be added to
   *  the index.
   **************************************************************** */
    Element e = makeNewEntry(name, fileName);
    int r = findInsertPoint(name, 1);
    if (r>=0) {
      insertEntry(e, r);
    }
  }

// --------- private methods --------------------------------------
/*
  private void newDfd(String filename, String dfd, Body mdb) {
    Document d = getNewDocument();
    Element root = d.createElement("MathDef1.0");
    root.appendChild(mdb.output(d));
    d.appendChild(root);
    writeXML(d,filename);
    recordIndexEntry(dfd,filename);
    saveIndex();
  }
  
  private void newArity(String filename, Body mdb, Document d) {
    Element root = (Element) d.getFirstChild();
    root.appendChild(mdb.output(d));
    writeXML(d,filename);
  }
  
  private void overwriteArity(String filename, Body mdb, Document d, Element e) {
    Element root = (Element) d.getFirstChild();
    root.replaceChild(mdb.output(d),e);
    writeXML(d,filename);
  }
*/
  private void insertEntry(Element newEntry, int r) {
  /** ***************************************************************
   *  Utility used by recordIndexEntry.
   **************************************************************** */
    NodeList entries = index.getFirstChild().getChildNodes();
    if (r == entries.getLength()/2) {
      index.getFirstChild().appendChild(newEntry);
      index.getFirstChild().appendChild( index.createTextNode("\n") );
    } else {
      index.getFirstChild().insertBefore(index.createTextNode("\n"),entries.item(2*r+1));
      entries = index.getFirstChild().getChildNodes();
      index.getFirstChild().insertBefore(newEntry,entries.item(2*r+1));
    }
  }
  
  private int findInsertPoint(String find, int column)
  {
  /** ***************************************************************
   *  Utility used by recordIndexEntry.
   **************************************************************** */
      NodeList entries = index.getFirstChild().getChildNodes();
      Node entry;
      NodeList row;
      String test = "";
      int r = entries.getLength()/2;
      for (int i = 0; i < entries.getLength()/2; i++) {
          entry = entries.item(2*i+1);
          row = entry.getChildNodes();
          test = row.item(column).getFirstChild().getNodeValue().trim();
          //System.out.println(test); //diagnostic
          int c = find.compareTo(test);
          //System.out.println("c = " + Integer.toString(c)); //diagnostic
          if (c <= 0) {
            if (c == 0 && !overwrite) {
              r = -1;
              break;
            } else {
              r = i;
              break;
            }
          }
      }
      return r;
  }
  
  private Element makeNewEntry(String name, String fileName) {
  /** ***************************************************************
   *  Utility used by recordIndexEntry.
   **************************************************************** */
    Element e;
    Element newEntry = (Element) index.createElement("entry");
    newEntry.appendChild( index.createTextNode(" ") );
    e = (Element) index.createElement("name");
    e.appendChild( index.createTextNode(name) );
    newEntry.appendChild(e);
    newEntry.appendChild( index.createTextNode(" ") );
    e = (Element) index.createElement("file");
    e.appendChild( index.createTextNode(fileName) );
    newEntry.appendChild(e);
    newEntry.appendChild( index.createTextNode(" ") );
    return newEntry;
  }
  
  private Document parseFileName(String fileName) {
  /** ***************************************************************
   *  Tries to open the xml file by the passed fileName
   *  (looking in the directory this.directory), parse it,
   *  and return the resulting Document. Returns null if this fails.
   **************************************************************** */
    Document result = null;
    try {
      result = builder.parse(new File(directory + fileName));
    } catch (SAXException sxe) {
      // Error generated during parsing
      Exception x = sxe;
      if (sxe.getException() != null)
        x = sxe.getException();
      x.printStackTrace();
    } catch (IOException ioe) {
      // I/O error
      ioe.printStackTrace();
    }
    return result;
  }
  
  private Document parseFile(File f) {
  /** ***************************************************************
   *  Tries to open the passed xml file, parse it,
   *  and return the resulting Document. Returns null if this fails.
   **************************************************************** */
    Document result = null;
    try {
      result = builder.parse(f);
    } catch (SAXException sxe) {
      // Error generated during parsing
      Exception x = sxe;
      if (sxe.getException() != null)
        x = sxe.getException();
      x.printStackTrace();
    } catch (IOException ioe) {
      // I/O error
      ioe.printStackTrace();
    }
    return result;
  }
  
  private String searchIndex(int columnIn, String s, int columnOut) {
  /** ***************************************************************
   *  Searches the index for an entry having String s
   *  in column number columnIn. Returns null if the String is not
   *  found; returns the entry in column number columnOut if it is.
   **************************************************************** */
      NodeList entries = index.getFirstChild().getChildNodes();
      Node entry;
      NodeList row;
      String t;
      for (int i = 0; i < entries.getLength()/2; i++) {
          entry = entries.item(2*i+1);
          row = entry.getChildNodes();
          t = row.item(columnIn).getFirstChild().getNodeValue().trim();
          //System.out.println(t); //diagnostic
          if ( t.equals(s) )
              return row.item(columnOut).getFirstChild().getNodeValue().trim();
      }
      return null;
  }
  
  private Document openIndex() {
  /** ***************************************************************
   *  Opens the index file.
   *  Builds a Document from this, and returns this Document.
   **************************************************************** */
    Document index = null;
    try {
      File f = new File(this.indexFile);
      index = builder.parse(f);
    } catch (SAXException sxe) {
      // Error generated during parsing
      Exception  x = sxe;
      if (sxe.getException() != null)
        x = sxe.getException();
      x.printStackTrace();
    } catch (IOException ioe) {
      // I/O error
      ioe.printStackTrace();
    } catch (Exception e) {
      // Search string was not found
      e.printStackTrace();
    }
    return index;
  }

// ----------------------------------------------------------------------------
}  // ----------------------------------------------------------------------
// ---------------------------------------------------------------------------- 
