 

import java.util.HashMap;

public class NLSet extends NLTerm {

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------

  NLTerm s;
  NLInfixrel alpha;
  NLFormula psi;
  NLTerm[] tt,uu,vv;
  
  String translation = "";
  boolean isInWords = false;
  
  boolean diag = false;

// ----------------------------------------------------------------------------
// Constructors
// ----------------------------------------------------------------------------

  public NLSet(NLTerm s, NLTerm[] tt, NLInfixrel alpha,
               NLTerm[] uu, NLFormula psi, NLTerm[] vv
  ) {
    this.s = s; this.tt = tt; this.alpha = alpha;
    this.uu = uu; this.psi = psi; this.vv = vv;
  }

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------
  
  public String xlat(int mode) {
    //First call the xlat methods of the children, and figure out
    //if we are forced to return words.
    //Start with the term s.
    s.xlat(mode);
    this.isInWords |= s.isInWords();
//diag:
  if (diag) {
    System.out.printf(
      "s is %s, and this %s in words.%n",
      s.getString(), (s.isInWords()?"is":"is not")
    );
  }
//
    //Now the term array tt, if it is not null.
    if (tt!=null) {
      for (int i = 0; i < tt.length; i++) {
        tt[i].xlat(mode); this.isInWords |= tt[i].isInWords();
//diag:
  if (diag) {
    System.out.printf(
      "tt[%d] is %s, and this %s in words.%n",
      i,
      tt[i].getString(), (tt[i].isInWords()?"is":"is not")
    );
  }
//
      }
    }
    //Treat the prepositional alpha, if it is there.
    if (alpha!=null) {
      boolean prepWords = s.isInWords();
      if (uu==null) {System.err.println("Prep. infix w/o RHS!");} else {
        for (int i = 0; i < uu.length; i++) {
          uu[i].xlat(mode); prepWords |= uu[i].isInWords();
//diag:
  if (diag) {
    System.out.printf(
      "uu[%d] is %s, and this %s in words.%n",
      i,
      uu[i].getString(), (uu[i].isInWords()?"is":"is not")
    );
  }
//
        }
      }
      if (prepWords || mode==NLPhrase.WORDS_MODE) {
        alpha.xlat(NLInfixrel.QUALIFIER_CODE);
      }
      else {alpha.xlat(NLInfixrel.DEFAULT_CODE);}
      prepWords |= alpha.isInWords();
//diag:
  if (diag) {
    System.out.printf(
      "alpha is %s, and this %s in words.%n",
      alpha.getString(), (alpha.isInWords()?"is":"is not")
    );
  }
//
      this.isInWords |= prepWords;
    }
    //Now psi, and the fixed variables, if they are there.
    if (psi!=null) {
      psi.xlat(mode); this.isInWords |= psi.isInWords();
//diag:
  if (diag) {
    System.out.printf(
      "psi is %s, and this %s in words.%n",
      psi.getString(), (psi.isInWords()?"is":"is not")
    );
  }
//
      if (vv!=null) {
        for (int i = 0; i < vv.length; i++) {
          vv[i].xlat(mode); //Variables cannot be in words.
        }
      }
    }
    //Finally, write the string.
    String x = "";
    if (this.isInWords || mode==NLPhrase.WORDS_MODE) {
      x = writeWords();
    } else {
      x = writeSymbols();
    }
    this.translation = x;
    return x;
  }

  public boolean isInWords() {
  //Don't call until *after* calling xlat.
    return isInWords;
  }

  public String getString() {return translation;}
  
  String writeSymbols() {
  //If you've settled on writing symbols, call this method to
  //produce the string.
    String x = "";
    x += "\\set{"+s.getString();
    if (tt!=null) {
      for (int i = 0; i < tt.length; i++) {
        x += ","+tt[i].getString();
      }
      x += "}";
    } else {
      if (alpha!=null) {
        x += alpha.getString();
        for (int i = 0; i < uu.length; i++) {
          x += (i>0?",":"")+uu[i].getString();
        }
      }
      if (psi!=null) {
        x += " : "+psi.getString();
      }
      if (vv!=null) {
        x += ", ";
        for (int i = 0; i < vv.length; i++) {
          x += (i>0?",":"")+vv[i].getString();
        }
        x += " fixed";
      }
      x += "}";
    }
    x = Evaluable.ddStrip(x);
    return "$"+x+"$";
  }
  
  String writeWords() {
  //If you've settled on writing words, call this method to
  //produce the string.
    String x = "";
    x += "the set ";
    if (tt!=null) {
      x += "whose elements are "+s.getString();
      for (int i = 0; i < tt.length; i++) {
        x += ", "+(i==tt.length-1?"and ":"")+tt[i].getString();
      }
    } else {
      //x += "of all ";
      x += "of ";
      x += s.getString();
      if (alpha!=null) {
        x += alpha.getString();
        for (int i = 0; i < uu.length; i++) {
          x += (i>0?",":"")+uu[i].getString();
        }
      }
      x += " such that "+psi.getString();
    }
    return x;
  }


// ----------------------------------------------------------------------------
}  // ----------------------------------------------------------------------
// ---------------------------------------------------------------------------- 






