%{
#include "string.h"
#include "yystype.h"
#include "yygrammar.h"

%}
%%
" "     { /* skip blank */ }
"\t"    { /* skip tab */ }
"\n"    { yypos++; /* adjust linenumber and skip newline */ }
"."     { return '.'; }
";"     { return ';'; }
"<"     { return '<'; }
">"     { return '>'; }
"-"     { return '-'; }
"["     { return '['; }
"]"     { return ']'; }
"0"     { return '0'; }
"("     { return '('; }
")"     { return ')'; }
"{"     { return '{'; }
"}"     { return '}'; }
","     { return ','; }
":"     { return ':'; }
"!"     { return '!'; }
"="     { return '='; }
"'"     { return '\''; }
"*"     { return '*'; }
"_"     { return '_'; }
"-ary"      { return ARY; }
"\\bot"     { return BOT; }
"DEFINITION"    { return DEFINITION; }
"\\dn"      { return DOWN; }
"\\exists"  { return EXISTS; }
"fixed"     { return FIXED; }
"\\forall"  { return FORALL; }
"function"  { return FUNC; }
"If"        { return IF; }
"\\iff"     { return IFF; }
"Infix"     { return INFIX; }
"\\infixfn" { return INFIXFN; }
"\\infixrl" { return INFIXRL; }
"\\lambda"  { return LAMBDA; }
"\\neg"     { return NEG; }
"\\neq"     { return NEQ; }
"Otherwise" { return OTHERWISE; }
"Precedence"    { return PRECEDENCE; }
"relation"  { return RELN; }
"\\simeq"   { return SIMEQ; }
"then"      { return THEN; }
"\\up"      { return UPARROW; }
"\\implies" { return THENSYM; }
"\\top"     { return TOP; }
"\\vee"     { return VEE; }
"\\wedge"   { return WEDGE; }
"\\mathbf"       { yylval=strdup(yytext); return FONT; }
"\\mathcal"      { yylval=strdup(yytext); return FONT; }
"\\mathfrak"     { yylval=strdup(yytext); return FONT; }
"\\mathscr"      { yylval=strdup(yytext); return FONT; }
"\\mathbb"      { return MATHBB; }
"\\hat"     { yylval=strdup(yytext); return DECORATION; }
"\\tilde"   { yylval=strdup(yytext); return DECORATION; }
"\\bar"     { yylval=strdup(yytext); return DECORATION; }
"\\vec"     { yylval=strdup(yytext); return DECORATION; }
"\\dot"     { yylval=strdup(yytext); return DECORATION; }
"\\alpha"   { yylval=strdup(yytext); return GREEK; }
"\\beta"    { yylval=strdup(yytext); return GREEK; }
"\\gamma"   { yylval=strdup(yytext); return GREEK; }
"\\delta"   { yylval=strdup(yytext); return GREEK; }
"\\epsilon" { yylval=strdup(yytext); return GREEK; }
"\\varepsilon"  { yylval=strdup(yytext); return GREEK; }
"\\veps"    { yylval=strdup(yytext); return GREEK; }
"\\zeta"    { yylval=strdup(yytext); return GREEK; }
"\\eta"     { yylval=strdup(yytext); return GREEK; }
"\\theta"   { yylval=strdup(yytext); return GREEK; }
"\\vartheta"    { yylval=strdup(yytext); return GREEK; }
"\\iota"    { yylval=strdup(yytext); return GREEK; }
"\\kappa"   { yylval=strdup(yytext); return GREEK; }
"\\mu"      { yylval=strdup(yytext); return GREEK; }
"\\nu"      { yylval=strdup(yytext); return GREEK; }
"\\xi"      { yylval=strdup(yytext); return GREEK; }
"\\omicron" { yylval=strdup(yytext); return GREEK; }
"\\pi"      { yylval=strdup(yytext); return FUNCSYM; }
"\\varpi"   { yylval=strdup(yytext); return GREEK; }
"\\rho"     { yylval=strdup(yytext); return GREEK; }
"\\varrho"  { yylval=strdup(yytext); return GREEK; }
"\\sigma"   { yylval=strdup(yytext); return GREEK; }
"\\varsigma"    { yylval=strdup(yytext); return GREEK; }
"\\tau"     { yylval=strdup(yytext); return GREEK; }
"\\upsilon" { yylval=strdup(yytext); return GREEK; }
"\\phi"     { yylval=strdup(yytext); return GREEK; }
"\\varphi"  { yylval=strdup(yytext); return GREEK; }
"\\vphi"    { yylval=strdup(yytext); return GREEK; }
"\\chi"     { yylval=strdup(yytext); return GREEK; }
"\\psi"     { yylval=strdup(yytext); return GREEK; }
"\\omega"   { yylval=strdup(yytext); return FUNCSYM; }
"\\Gamma"   { yylval=strdup(yytext); return GREEK; }
"\\Delta"   { yylval=strdup(yytext); return GREEK; }
"\\Theta"   { yylval=strdup(yytext); return GREEK; }
"\\Lambda"  { yylval=strdup(yytext); return GREEK; }
"\\Xi"      { yylval=strdup(yytext); return GREEK; }
"\\Pi"      { yylval=strdup(yytext); return GREEK; }
"\\Sigma"   { yylval=strdup(yytext); return GREEK; }
"\\Upsilon" { yylval=strdup(yytext); return GREEK; }
"\\Phi"     { yylval=strdup(yytext); return GREEK; }
"\\Psi"     { yylval=strdup(yytext); return GREEK; }
"\\Omega"   { yylval=strdup(yytext); return GREEK; }
"+"         { yylval=strdup(yytext); return FUNCSYM; }
"^"         { yylval=strdup(yytext); return FUNCSYM; }
"\\less"    { yylval=strdup(yytext); return FUNCSYM; }
"/"         { yylval=strdup(yytext); return FUNCSYM; }
"|"         { yylval=strdup(yytext); return FUNCSYM; }
"\\cup"     { yylval=strdup(yytext); return FUNCSYM; }
"\\cap"     { yylval=strdup(yytext); return FUNCSYM; }
"\\sqcap"   { yylval=strdup(yytext); return FUNCSYM; }
"\\sqcup"   { yylval=strdup(yytext); return FUNCSYM; }
"\\oplus"   { yylval=strdup(yytext); return FUNCSYM; }
"\\otimes"  { yylval=strdup(yytext); return FUNCSYM; }
"\\times"   { yylval=strdup(yytext); return FUNCSYM; }
"\\div"     { yylval=strdup(yytext); return FUNCSYM; }
"\\circ"    { yylval=strdup(yytext); return FUNCSYM; }
"\\cdot"    { yylval=strdup(yytext); return FUNCSYM; }
"\\varnothing"  { yylval=strdup(yytext); return FUNCSYM; }
"\\infty"   { yylval=strdup(yytext); return FUNCSYM; }
"\\wp"       { yylval=strdup(yytext); return FUNCSYM; }
"\\sum"     { yylval=strdup(yytext); return FUNCSYM; }
"\\prod"    { yylval=strdup(yytext); return FUNCSYM; }
"\\coprod"  { yylval=strdup(yytext); return FUNCSYM; }
"\\bigcap"  { yylval=strdup(yytext); return FUNCSYM; }
"\\bigcup"  { yylval=strdup(yytext); return FUNCSYM; }
"\\bigsqcup"    { yylval=strdup(yytext); return FUNCSYM; }
"\\bigvee"  { yylval=strdup(yytext); return FUNCSYM; }
"\\bigwedge"    { yylval=strdup(yytext); return FUNCSYM; }
"\\bigotimes"   { yylval=strdup(yytext); return FUNCSYM; }
"\\bigoplus"    { yylval=strdup(yytext); return FUNCSYM; }
"\\aleph"   { yylval=strdup(yytext); return FUNCSYM; }
"\\beth"    { yylval=strdup(yytext); return FUNCSYM; }
"\\gimel"   { yylval=strdup(yytext); return FUNCSYM; }
"\\daleth"  { yylval=strdup(yytext); return FUNCSYM; }
"\\Re"      { yylval=strdup(yytext); return FUNCSYM; }
"\\Im"      { yylval=strdup(yytext); return FUNCSYM; }
"\\partial" { yylval=strdup(yytext); return FUNCSYM; }
"\\nabla"   { yylval=strdup(yytext); return FUNCSYM; }
"\\leq"     { yylval=strdup(yytext); return RELNSYM; }
"\\geq"     { yylval=strdup(yytext); return RELNSYM; }
"\\prec"    { yylval=strdup(yytext); return RELNSYM; }
"\\succ"    { yylval=strdup(yytext); return RELNSYM; }
"\\preceq"  { yylval=strdup(yytext); return RELNSYM; }
"\\succeq"  { yylval=strdup(yytext); return RELNSYM; }
"\\ll"  { yylval=strdup(yytext); return RELNSYM; }
"\\gg"  { yylval=strdup(yytext); return RELNSYM; }
"\\subset"  { yylval=strdup(yytext); return RELNSYM; }
"\\supset"  { yylval=strdup(yytext); return RELNSYM; }
"\\subseteq"    { yylval=strdup(yytext); return RELNSYM; }
"\\supseteq"    { yylval=strdup(yytext); return RELNSYM; }
"\\sqsubseteq"  { yylval=strdup(yytext); return RELNSYM; }
"\\sqsupseteq"  { yylval=strdup(yytext); return RELNSYM; }
"\\in"  { yylval=strdup(yytext); return RELNSYM; }
"\\ni"  { yylval=strdup(yytext); return RELNSYM; }
"\\vdash"   { yylval=strdup(yytext); return RELNSYM; }
"\\dashv"   { yylval=strdup(yytext); return RELNSYM; }
"\\equiv"   { yylval=strdup(yytext); return RELNSYM; }
"\\models"  { yylval=strdup(yytext); return RELNSYM; }
"\\sim" { yylval=strdup(yytext); return RELNSYM; }
"\\perp"    { yylval=strdup(yytext); return RELNSYM; }
"\\mid" { yylval=strdup(yytext); return RELNSYM; }
"\\asymp"   { yylval=strdup(yytext); return RELNSYM; }
"\\parallel"    { yylval=strdup(yytext); return RELNSYM; }
"\\approx"  { yylval=strdup(yytext); return RELNSYM; }
"\\smile"   { yylval=strdup(yytext); return RELNSYM; }
"\\cong"    { yylval=strdup(yytext); return RELNSYM; }
"\\frown"   { yylval=strdup(yytext); return RELNSYM; }
"\\propto"  { yylval=strdup(yytext); return RELNSYM; }
"\\doteq"   { yylval=strdup(yytext); return RELNSYM; }
"\\bowtie"  { yylval=strdup(yytext); return RELNSYM; }
"\\lhd" { yylval=strdup(yytext); return RELNSYM; }
"\\rhd" { yylval=strdup(yytext); return RELNSYM; }
"\\unlhd"   { yylval=strdup(yytext); return RELNSYM; }
"\\unrhd"   { yylval=strdup(yytext); return RELNSYM; }
"\\ell" { yylval=strdup(yytext); return MISCLETTER; }
[1-9][0-9]*   { yylval=strdup(yytext); return NZNUM; }
[0-9]+  { yylval=strdup(yytext); return DIGITS; }
[A-Za-z] { yylval=strdup(yytext); return LETTER; }
[A-Z][a-z]+ { yylval=strdup(yytext); return INITCAP; }
[A-Z][A-Z]+ { yylval=strdup(yytext); return ALLCAPS; }
[A-Za-z]+ { yylval=strdup(yytext); return LETTERS; }
[A-Za-z0-9]+    { yylval=strdup(yytext); return GENWORD; }
